% Gets the fitting cost (i.e., the difference between the value function
% approximation and the actual value function).
%
% Arguments:
%
% w             weights
function fitting_cost = get_fitting_cost(w, s1, s2, num_bases, x1_min, x1_max, x2_min, x2_max, n1, n2, a, b, r, k)
    w = abs(w);

    x1_vals = linspace(x1_min, x1_max, n1);
    x2_vals = linspace(x2_min, x2_max, n2);    

    % Get approximate value with value function (values is a (n1)x(n2)
    % matrix where (i, j) indexes into the value for the ith x1 and jth x2)
    values = get_opt_values(w, s1, s2, num_bases, x1_min, x1_max, x2_min, x2_max, n1, n2);

    % Calculate partial derivatives for x1 and x2 (used in Bellman equation
    % calculation)
    partials_x1 = diff(values,1,1);
    partials_x2 = diff(values,1,2);
    % Pad partial matrices to make the same size as values
    partials_x1 = padarray(partials_x1, [1 1], 'post', 'replicate');
    partials_x2 = padarray(partials_x2, [1 1], 'post', 'replicate');
    
%     disp(w);
    
    
    % Use approximate value to calculate Bellman equation T[v]

    T = zeros(n1, n2);
    
    for i=1:n1,
       for j=1:n2,
            x1 = x1_vals(i);
            x2 = x2_vals(j);    
            term1 = x2 * partials_x1(i,j);
            term2 = (a*sin(x1)-b*x2)*partials_x2(i,j);
            term3 = -(1/(2*r))*(partials_x2(i,j)).^2;
            T(i,j) = term1 + term2 + term3 + 1-exp(k*cos(x1)-k);
       end
    end

    % Finally, calculate fitting cost
    fitting_cost = sum(norm(values - T).^2);
%     disp('Fitting cost return value:');
%     disp(fitting_cost);
end