% Finds all of the neighboring states of y within x_vals.
%
% Arguments:
%
% y             output value (we're trying to find its neighbors)
% x_vals        all of the values on this state axis
% wrap          whether or not to wrap around at the end of the state range
function neighbors = get_neighbors(y, x_vals, wrap)
    x_max = x_vals(end);

    neighbors = struct;
    
    % Handle case where y is already in the state space
    if(~isempty(find(x_vals==y,1))),
        neighbors.points = y;
        neighbors.indices = find(x_vals==y,1);
        neighbors.probs = 1;
        return
    end
    
    % Handle case where y is past the bounds of the state space
    % by reducing y to the bounds of [0, x_max]
    if wrap,
        y = mod(y, x_max);
    end

    if (y < -x_max),
        neighbors.points = -x_max;
        neighbors.indices = 1;
        neighbors.probs = 1;
        return
    elseif (y > x_max),
        neighbors.points = x_max;
        neighbors.indices = [length(x_max)];
        neighbors.probs = 1;
        return
    end    

    % Find two nearest neighbors on the state axis
    [neighbor_indices, neighbor_distances] = knnsearch(x_vals',y,'k',2);

    % Assign neighbor probabilities according to distance
    neighbors.points = [x_vals(neighbor_indices(1)) x_vals(neighbor_indices(2))];
    neighbors.indices = neighbor_indices;
    total_distance = sum(neighbor_distances);
    neighbors.probs = [1-neighbor_distances(1)/total_distance 1-neighbor_distances(2)/total_distance];
    
end