% Gets the approximated optimal value function.
%
% Arguments:
%
% w             A vector of N weights for each expression.
% s1            Base sharpness for x1 part of state.
% s2            Base sharpness for x2 part of state.
% num_bases     Number of bases on EACH of the x1 and x2 axes.
% x1_min        Minimum x1 value.
% x1_max        Maximum x1 value.
% x2_min        Minimum x2 value.
% x2_max        Maximum x2 value.
% n1         # of grid points on state x1 (arc) dimension
% n2         # of grid points on state x2 (velocity) dimension
function opt_values = get_opt_values(w, s1, s2, num_bases, x1_min, x1_max, x2_min, x2_max, n1, n2)

%     w_sum = sum(w);

    x1_vals = linspace(x1_min, x1_max, n1);
    x2_vals = linspace(x2_min, x2_max, n2);

    % Centers of position bases
    c1 = linspace(x1_min, x1_max, num_bases);
    c2 = linspace(x1_min, x1_max, num_bases);

    % opt_values is a (n1)x(n2) matrix where (i, j) indexes into the value
    % for the ith x1 and jth x2
    opt_values = zeros(n1, n2);
    for i=1:n1,
       for j=1:n2,
           for base=1:num_bases,
               x1 = x1_vals(i);
               x2 = x2_vals(j);

               term1 = -s1*cos(x1-c1(base));
               term2 = -s2*(x2-c2(base)).^2;

               w_sum = sum(w);
               opt_values(i,j) = opt_values(i,j) + w(base) .* exp(term1 + term2);
               
           end
       end
    end
end