% Given an x1 index and x2 index, looks in the y1_mat and y2_mat transition
% matrices and finds out what the next states are, then figures out those
% states' values. Takes the minimum value, and finds the input that would
% be necessary to get to the corresponding state. Returns that input.
function policy = get_policy(x1_idx, x2_idx, u_vals, y1_mat, y2_mat, opt_values)

    alpha = 0.5;

    % Iterate through possible inputs, and take the one with the best
    % hamiltonian (of the estimated value of the next state)
    
    hamiltonians = zeros(1,length(u_vals));
    
    for u_idx = 1:length(u_vals),
        
        % Sum probability-weighted values for each
        % action, based on what next states they can
        % take you to
        next1.points = y1_mat(x1_idx,x2_idx,u_idx).points;
        next1.indices = y1_mat(x1_idx,x2_idx,u_idx).indices;
        next1.probs = y1_mat(x1_idx,x2_idx,u_idx).probs;
        next2.points = y2_mat(x1_idx,x2_idx,u_idx).points;
        next2.indices = y2_mat(x1_idx,x2_idx,u_idx).indices;
        next2.probs = y2_mat(x1_idx,x2_idx,u_idx).probs;

        estimated_value_sum = 0;
        for n1_i = 1:length(next1.indices),
            for n2_i = 1:length(next2.indices),
                this_state_value = opt_values(next1.indices(n1_i), next2.indices(n2_i));
                this_state_prob = next1.probs(n1_i) * next2.probs(n2_i);
                estimated_value_sum = estimated_value_sum + this_state_value * this_state_prob;
            end
        end        
                
        hamiltonians(u_idx) = opt_values(x1_idx,x2_idx) + alpha*estimated_value_sum;
      
    end
    [value, policy] = min(hamiltonians);
    
      


%                 % Sum probability-weighted values for each
%                 % action, based on what next states they can
%                 % take you to
%                 next1.points = y1_mat(x1_idx,x2_idx,u_idx).points;
%                 next1.indices = y1_mat(x1_idx,x2_idx,u_idx).indices;
%                 next1.probs = y1_mat(x1_idx,x2_idx,u_idx).probs;
%                 next2.points = y2_mat(x1_idx,x2_idx,u_idx).points;
%                 next2.indices = y2_mat(x1_idx,x2_idx,u_idx).indices;
%                 next2.probs = y2_mat(x1_idx,x2_idx,u_idx).probs;
%                 next1.input
% 
%                 % Make a small (4x4) matrix of weighted values, then take
%                 % the minimum value. Each row is a next x1 value; each
%                 % column is a next x2 value.
%                 val_mat = ones(2, 2);
%                 val_mat(1, 1) = opt_values(next1.indices(1), next2.indices(1)) * next1.probs(1) * next2.probs(1);
%                 if length(next1.indices) == 2,
%                     val_mat(2, 1) = opt_values(next1.indices(2), next2.indices(1)) * next1.probs(2) * next2.probs(1);
%                 end
%                 if length(next2.indices) == 2,
%                     val_mat(1, 2) = opt_values(next1.indices(1), next2.indices(2)) * next1.probs(1) * next2.probs(2);
%                 end
%                 if (length(next1.indices) == 2)&&(length(next2.indices) == 2),
%                     val_mat(2, 2) = opt_values(next1.indices(2), next2.indices(2)) * next1.probs(2) * next2.probs(2);
%                 end
%                 
%                 % Find minimum value
%                 % finds the min of val_mat and its position, when val_mat is viewed as a 1D array
%                 [min_val, position] = min(val_mat(:)); 
% 
%                 % transform the index in the 1D view to 2 indices, given the size of val_mat
%                 [next_x1_to_choose, next_x2_to_choose] = ind2sub(size(val_mat),position);
% 
%                 % Get input corresponding to this choice
%                 x1_chosen = next1.indices(next_x1_to_choose);
%                 x2_chosen = next2.indices(next_x2_to_choose);
                

end